/*
 * HSimulator
 * Copyright 2015 Luca Marchetti
 * 
 * This file is part of HSimulator.
 * 
 * HSimulator is distributed according to the Cosbi Shared Source license 
 * agreement ("COSBI-SSLA"). Please, refer to the documentation available
 * at http://www.cosbi.eu for any detail.
 */

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.concurrent.ExecutionException;
import hSimulator.library.Model;
import hSimulator.library.SimulatedTS;
import hSimulator.library.SimulationStoppedException;
import hSimulator.library.Simulator;

/**
 * This class provides an example of usage of HSimulator.
 * The model Gemcitabine will be loaded and simulated.
 * Please, be sure that the file <code>Gemcitabine.txt</code> storing the model
 * specification is stored in the same folder of this class.<br />
 * <br />
 * To compile: <code>javac -classpath HSimulator.jar Gemcitabine.java</code><br />
 * To execute (Linux): <code>java -classpath .:HSimulator.jar Gemcitabine</code><br />
 * To execute (Windows): <code>java -classpath .;HSimulator.jar Gemcitabine</code>
 *
 * @author Luca Marchetti
 * @version 1.0
 */
public class Gemcitabine {
    
    /**
     * Main method.
     * 
     * @param args  array of arguments
     */
    public static void main(String[] args) {

        // loading from file of the specification
        String modelSpec = readSpecification();
        
        // creation of the model instance
        Model m = null;
        try {
            m = new Model(modelSpec); // the constructor parses the specification and creates the model instance
        } catch (ParseException e) {
            // if something is wrong during the parsing of the model specification,
            // a suitable ParseException is thrown. A description of the error is
            // provided in the exception message.
            System.err.println("Fatal error during the parsing of the model specification.\n"+e.getMessage());
        }
        
        // creation of the model simulator
        Simulator s = new Simulator(m);
        
        // running a simulation:
        // the simulator supports 5 algorithms:
        // 1) Gillespie DM -> methods simDM(...)
        // 2) RSSA ---------> methods simRSSA(...)
        // 3) Euler --------> methods simEuler(...)
        // 4) RK45 ---------> methods simRK45(...)
        // 5) HRSSA --------> methods simHRSSA(...)
        // please, look at the JavaDoc documentation for details
        
        // here we simulate the model by means of HRSSA
        // simulation parameters of HRSSA
        int nSim = 50; //number of parallel simulations that will be computed
        int tMax = 24; // maximum time instant to be simulated (24 time units)
        double delta = 0.1; // delta for calculating the fluctuation interval of variable states
        int epsilon = 10;  // minimum size of fluctuation intervals
        int gamma = 100; // minimum molecule number of fast reactions
        int theta = 10; // minimum number of reaction firing of fast reactions
        double dT = 0.01; // discretization step used to simulate fast reactions
        double tsSampling = 0.1; // the time sampling used for storing time series during the simulation
        
		try {
			// running HRSSA
			// the following call runs 50 HRSSA simulations using multithreading
			// finally, simulation results are averaged and stored in a SimulatedTS object.
			// During the simulation process, it is possible to inspect the percentage of
			// completed calculations using the method s.getCompletedPercentage()
			System.out.println("Starting 50 HRSSA simulations...");
			SimulatedTS ts = s.simHRSSA(nSim, tMax, delta, epsilon, gamma, theta, dT, tsSampling);
			
			// IMPORTANT: the previous call stores in ts ALL the time series of model variables.
			// If you don't need this, you can specify in a list the IDs of variables that you want to preserve (see JavaDoc)
			// Please, remember that the ID of a variable is given by: variableName@CompartmentName.
			// For example, the ID of variable dFdC_out in compartment System is: dFdC_out@System
			
			// DURING SIMULATION: you can prematurely stop a simulation by means of the method s.stopSimulation()
			// a simulation process can be also paused and then resumed by means of s.pauseSimulation() and s.resumeSimulation()
			
			// looking at simulation results
			// the SimulatedTS object provides a lot of methods for looking at simulation results
			// here we gives some examples, please refer to the JavaDoc documentation for a complete list of provided methods

			// to work on simulated time series
			double[] timeTS = ts.getTimeTs(); // array storing time instants
			double[][] dynamics = ts.getDynamicsTs(); // matrix storing the time series of all the model variables
			double[] dynamics2 = ts.getDynamicsTs("dFdC_out@System"); // time series of variable dFdC_out in compartment System
			
			// this calls returns the averaged computation time for a simulation
			System.out.println("Averaged computation time: "+ ts.getComputationTime() +" sec.");
			
			// this call plots simulation results using JFreeChart
			ts.plotSimulation("Gemcitabine (HRSSA)");  // if you want to add the JFreeChart object to your frame, you can use the method getSimulationPlot()
			
			// this call exports simulation results in a MS Excel document using JExcelAPI
			try {
				ts.exportToExcel("Gemcitabine.xls");
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		} catch (ExecutionException ex) {
			// this type of exception is raised when an error occur during the simulation
			System.err.println(ex.getCause().getMessage());
		} catch (SimulationStoppedException ex) {
			// this type of exception is raised when the user stopped the simulation process prematurely
			// this can be done by means of the method stopSimulation() of the object Simulator
			// partial simulation results can be obtained by the method: ex.getPartialResults()
			System.err.println(ex.getMessage());
		}

		// before ending, we can shut down the simulator
		s.shutDown(); // this shuts down the Java service for parallel computing
	}
    
    /**
     * Reads the model specification from file.
     * 
     * @return a <code>String</code> with the model specification
     */
    public static String readSpecification() {
        String spec = "";
        // trying to read from file the specification...
        try {
            BufferedReader reader = new BufferedReader(new FileReader("Gemcitabine.txt"));
            String line = reader.readLine();
            while(line!=null) {
                spec += line + "\n";
                line = reader.readLine();
            }        
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return spec;
    }
}
